package cz.fidentis.analyst.core;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;

/**
 * Builder for control panels. Layout is based on GridBagLayout. 
 * 
 * @author Radek Oslejsek
 */
public class ControlPanelBuilder {
    
    public static final Font CAPTION_FONT= new Font("Arial", 1, 18);
    public static final Insets CAPTION_PADDING = new Insets(20, 0, 20, 0);  // top, left, bottom, right
    public final ImageIcon HELP_ICON = new ImageIcon(getClass().getResource("/info.png"));
    public static final Font OPTION_TEXT_FONT = new Font("Arial", 1, 14);
    public static final Color OPTION_TEXT_COLOR = new java.awt.Color(20, 114, 105);
    public static final int OPTION_TEXT_WIDTH = 20; // number of cells
    public static final int SLIDER_WIDTH = 20; // number of cells
    public static final int BUTTON_WIDTH = 12; // number of cells
    
    private JPanel controlPanel;
    private int row = 0;
    private int col = 0;
    
    /**
     * Constructor.
     * 
     * @param controlPanel Control panel to which the GUI elements are inserted.
     */
    public ControlPanelBuilder(JPanel controlPanel) {
        if (controlPanel == null) {
            throw new IllegalArgumentException("controlPanel");
        }
        this.controlPanel = controlPanel;
        GridBagLayout layout = new GridBagLayout();
        this.controlPanel.setLayout(layout);
    }
    
    /**
     * Adds a line with caption.
     * 
     * @param caption Caption text.
     * @return This builder
     */
    public ControlPanelBuilder addCaptionLine(String caption) {
        GridBagConstraints c = new GridBagConstraints();
        c.insets = CAPTION_PADDING; 
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = col;
        c.gridy = row;
        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.NONE;
        JLabel label = new JLabel(caption);
        label.setFont(CAPTION_FONT);
        controlPanel.add(label, c);
        addLine();
        return this;
    }
    
    /**
     * Adds a line with slider option.
     * 
     * @param helpAction Action listener invoked when the help icon is clicked. If {@code null}, then no help is shown.
     * @param text Option text.
     * @param sliderMax Max value of the slider (and the value field). If {@code -1}, then percentage slider is shown with 100 as the max. value.
     * @param valueField Input field connected with the slider.
     * @return This builder
     */
    public ControlPanelBuilder addSliderOptionLine(ActionListener helpAction, String text, int sliderMax, JTextField valueField) {
        if (helpAction != null) {
            addOptionHelpIcon(helpAction);
        } else {
            col++;
        }
        addOptionText((text == null) ? "" : text);
        addSliderWithVal(sliderMax, valueField);
        addLine();
        return this;
    }
    
    /**
     * Adds a line with a checkbox option.
     * 
     * @param helpAction Action listener invoked when the help icon is clicked. If {@code null}, then no help is shown.
     * @param text Option text.
     * @param selected Initial state of the checkbox.
     * @param checkBoxAction Action listener invoked when the check box is clicked.
     * @return This builder
     */
    public ControlPanelBuilder addCheckBoxOptionLine(ActionListener helpAction, String text, boolean selected, ActionListener checkBoxAction) {
        if (helpAction != null) {
            addOptionHelpIcon(helpAction);
        } else {
            col++;
        }
        addOptionText((text == null) ? "" : text);
        addCheckBox(selected, checkBoxAction);
        addLine();
        return this;
    }

    /**
     * Moves the builder to the new line. Methods with "Line" suffix add new line automatically.
     * 
     * @return This builder
     */
    public ControlPanelBuilder addLine() {
        row++;
        col = 0; 
        return this;
    }
    
    /**
     * Adds a button section at the bottom of the control panel.
     * 
     * @param buttons Labels.
     * @param actions Action listener invoked when the corresponding button is clicked.
     * @return This builder
     * @throws IllegalArgumentException if the size of the two lists is different
     */
    public ControlPanelBuilder addButtons(List<String> buttons, List<ActionListener> actions) {
        if (buttons.size() != actions.size()) {
            throw new IllegalArgumentException();
        }
        
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(2, 0, 40, 0); // small external space on top and bottom
        c.weighty = 1.0;   //request any extra vertical space
        c.gridwidth = BUTTON_WIDTH;
        c.gridy = row;
        c.anchor = GridBagConstraints.PAGE_END;
        c.fill = GridBagConstraints.NONE;
        
        for (int i = 0; i < buttons.size(); i++) {
            JButton button = new JButton();
            button.setText(buttons.get(i));
            button.addActionListener(actions.get(i));
            c.gridx = col;
            col += BUTTON_WIDTH;
            controlPanel.add(button, c);
        }

        return this;
    }
    
    /**
     * Adds a check box.
     * 
     * @param selected Initial state
     * @param actions Action listener invoked when the checkbox is clicked.
     * @return This builder
     */
    public ControlPanelBuilder addCheckBox(boolean selected, ActionListener action) {
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(0, 25, 0, 0);
        c.gridwidth = 1;
        c.gridx = col++;
        c.gridy = row;
        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.NONE;

        JCheckBox checkBox = new JCheckBox();
        checkBox.setSelected(selected);
        checkBox.addActionListener(action);
        controlPanel.add(checkBox, c);

        return this;
    }
    
    /**
     * Adds a help icon.
     * 
     * @param actions Action listener invoked when the icon is clicked.
     * @return This builder
     */
    public ControlPanelBuilder addOptionHelpIcon(ActionListener action) {
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = 1;
        c.gridx = col++;
        c.gridy = row;
        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.NONE;
        
        JButton button = new JButton();
        button.setBorderPainted(false);
        button.setFocusPainted(false);
        button.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        button.addActionListener(action);
        button.setIcon(HELP_ICON);
        
        controlPanel.add(button, c);
        return this;
    }
    
    /**
     * Adds a text of the option.
     * 
     * @param text Text.
     * @return This builder
     */
    public ControlPanelBuilder addOptionText(String text) {
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = OPTION_TEXT_WIDTH;
        c.gridx = col;
        col += OPTION_TEXT_WIDTH;
        c.gridy = row;
        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.HORIZONTAL;
        
        JLabel label = new JLabel();
        label.setFont(OPTION_TEXT_FONT);
        label.setForeground(OPTION_TEXT_COLOR);
        label.setText(text);
        
        controlPanel.add(label, c);
        return this;
    }
    
    /**
     * Adds a slider and connects it with given input field
     * 
     * @param max Max value of the slider (and the value field). If {@code -1}, then percentage slider is shown with 100 as the max. value.
     * @param valueField Input field connected with the slider.
     * @return This builder
     */
    public ControlPanelBuilder addSliderWithVal(int max, JTextField valueField) {
        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = SLIDER_WIDTH;
        c.gridx = col;
        col += SLIDER_WIDTH;
        c.gridy = row;
        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.HORIZONTAL;

        JSlider slider = new JSlider();
        if (max == -1) {
            slider.setMaximum(100);
            slider.setValue((int) (Double.parseDouble(valueField.getText()) * 100));
        } else { // absolute values
            slider.setMaximum(max);
            slider.setValue((int) (Double.parseDouble(valueField.getText())));
        }
        slider.addChangeListener((ChangeEvent ce) -> {
            valueField.setText((max == -1) ? ""+slider.getValue()/100.0 : ""+slider.getValue());
            valueField.postActionEvent(); // invoke textField action listener
        });
        valueField.addActionListener((ActionEvent ae) -> {
            if (max == -1) {
                slider.setValue((int) (Double.parseDouble(valueField.getText()) * 100));
            } else {
                slider.setValue((int) (Double.parseDouble(valueField.getText())));
            }
            //setDefaultButton.setEnabled(true);
            //resetButton.setEnabled(true);
            //applyButton.setEnabled(true);
        });
        controlPanel.add(slider, c);
        
        c.gridwidth = 2;
        c.gridx = col;
        col += 2;
        c.gridy = row;
        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.BOTH;
        controlPanel.add(valueField, c);
        
        return this;
    }
    
}
