package cz.fidentis.analyst.symmetry;

import cz.fidentis.analyst.core.ControlPanelBuilder;
import java.awt.event.ActionEvent;
import java.util.List;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import org.openide.windows.WindowManager;

/**
 * Control panel for symmetry plane.
 *
 * @author Natalia Bebjakova
 * @author Radek Oslejsek
 */
public final class SymmetryPanel extends javax.swing.JPanel {
    
    public static final int BORDER_WIDTH = 10;
    public static final int ROW_HEIGHT = 40;
    public static final int MAX_SIGNIFICANT_POINTS = 500;
    
    public static final String ACTION_COMMAND_SHOW_SIGN_POINTS_HELP = "significant points help";
    public static final String ACTION_COMMAND_SHOW_MIN_CURV_HELP = "min curvature help";
    public static final String ACTION_COMMAND_SHOW_REL_DIST_HELP = "relative distance help";
    public static final String ACTION_COMMAND_SHOW_MIN_ANGLE_COS_HELP = "min angle cosinus help";
    public static final String ACTION_COMMAND_SHOW_NORMAL_ANGLE_HELP = "normal angle help";
    public static final String ACTION_COMMAND_SHOW_AVERAGING_HELP = "averaging help";    
    
    /**
     * Configuration with optional parameters of the algorithm 
     */
    private SymmetryConfig config;
    private final SymmetryPlaneAction spAction;
    
    // GUI primitives hoding the configuration state:
    private final JTextField signPointsTF = new JTextField();
    private final JTextField minCurvatureTF = new JTextField();
    private final JTextField minAngleCosTF = new JTextField();
    private final JTextField minNormAngleCosTF = new JTextField();
    private final JTextField maxRelDistTF = new JTextField();
    
    private final JCheckBox  averaging = new JCheckBox();
    private final JCheckBox  showSymmetryPlane = new JCheckBox();
    
    /**    
     * Constructor.
     * 
     * @param spAction Action listener
     */
    public SymmetryPanel(SymmetryPlaneAction spAction) {
        this.config = new SymmetryConfig();
        this.spAction = spAction;
        
        resetSymmetryParams(config);
        showSymmetryPlane.setSelected(true);
        
        new ControlPanelBuilder(this)
                .addCaptionLine("Computation options:")
                .addSliderOptionLine((ActionEvent e) -> { showSignPointsHelp();  }, "Significant points", MAX_SIGNIFICANT_POINTS, signPointsTF)
                .addSliderOptionLine((ActionEvent e) -> { showMinCurvHelp();     }, "Min. curvature ratio", -1, minCurvatureTF)
                .addSliderOptionLine((ActionEvent e) -> { showMinAngleCosHelp(); }, "Min. angle cosine", -1, minAngleCosTF)
                .addSliderOptionLine((ActionEvent e) -> { showNormalAngleHelp(); }, "Normal angle", -1, minNormAngleCosTF)
                .addSliderOptionLine((ActionEvent e) -> { showRelDistHelp();     }, "Relative distance", -1, maxRelDistTF)
                .addCheckBoxOptionLine((ActionEvent e) -> { showAveragingHelp(); }, "Averaging", averaging.isSelected(), 
                        (ActionEvent e) -> { 
                            averaging.setSelected(((JToggleButton) e.getSource()).isSelected()); 
                        })
                .addCaptionLine("Visualization options:")
                .addCheckBoxOptionLine(null, "Show symmetry plane", showSymmetryPlane.isSelected(), 
                        (ActionEvent e) -> { 
                            spAction.actionPerformed(new ActionEvent(
                                    e.getSource(), 
                                    ActionEvent.ACTION_PERFORMED, 
                                    SymmetryPlaneAction.ACTION_COMMAND_SHOW_HIDE_PLANE)
                            ); 
                        })
                .addButtons(
                        List.of("Reset changes", "Reset to defaults", "(Re)compute"), 
                        List.of(
                                (ActionEvent e) -> {
                                    resetSymmetryParams(config);
                                },
                                (ActionEvent e) -> {
                                    resetSymmetryParams(new SymmetryConfig());
                                },
                                (ActionEvent e) -> {
                                    spAction.actionPerformed(new ActionEvent(
                                            this, 
                                            ActionEvent.ACTION_PERFORMED, 
                                            SymmetryPlaneAction.ACTION_COMMAND_RECOMPUTE)
                                    ); 
                                }
                        )
                );
    }
    
    /**
     * Returns symmetry plane configuration
     * @return symmetry plane configuration
     */
    public SymmetryConfig getSymmetryConfig() {
        return this.config;
    }
    
    /**
     * Applies changes made in the control panel - copies them to the configuration object.
     */
    public void confirmConfigParams() {
        config.setMaxRelDistance(Double.parseDouble(this.maxRelDistTF.getText()));
        config.setMinAngleCos(Double.parseDouble(this.minAngleCosTF.getText()));
        config.setMinCurvRatio(Double.parseDouble(this.minCurvatureTF.getText()));
        config.setMinNormAngleCos(Double.parseDouble(this.minNormAngleCosTF.getText()));
        config.setSignificantPointCount(Integer.parseInt(this.signPointsTF.getText()));
        config.setAveraging(this.averaging.isSelected());
    }
    
    private void resetSymmetryParams(SymmetryConfig config) {
        this.signPointsTF.setText("" + config.getSignificantPointCount());
        this.minCurvatureTF.setText("" + config.getMinCurvRatio());
        this.minAngleCosTF.setText("" + config.getMinAngleCos());
        this.minNormAngleCosTF.setText("" + config.getMinNormAngleCos());
        this.maxRelDistTF.setText("" + config.getMaxRelDistance());
        this.averaging.setSelected(config.isAveraging());
        
        // fire changes:
        this.signPointsTF.postActionEvent();
        this.minCurvatureTF.postActionEvent();
        this.minAngleCosTF.postActionEvent();
        this.minNormAngleCosTF.postActionEvent();
        this.maxRelDistTF.postActionEvent();
    }
    
    private void showSignPointsHelp() {
        JOptionPane.showMessageDialog(WindowManager.getDefault().findTopComponent("SingleFaceTab"),
                "Entered number represents amount of points of the mesh that are taken into account" + System.lineSeparator()
                        + "while counting the plane of approximate symmetry." + System.lineSeparator() 
                        + System.lineSeparator()
                        + "Higher number → longer calculation, possibly more accurate result." + System.lineSeparator()
                        + "Lower number → shorter calculation, possibly less accurate result.", 
                "Significant points",
                0, 
                new ImageIcon(getClass().getResource("/points.png"))
        );
                
    }
     
    private void showMinAngleCosHelp() {
        JOptionPane.showMessageDialog(WindowManager.getDefault().findTopComponent("SingleFaceTab"),
                "Entered number represents how large the angle between normal vector of candidate plane and the vector" + System.lineSeparator()
                        + "of two vertices can be to take into account these vertices while counting the approximate symmetry."  + System.lineSeparator()
                        + System.lineSeparator()
                        + "Higher number → fewer pairs of vertices satisfy the criterion → shorter calculation, possibly less accurate result." + System.lineSeparator()
                        + "Lower number → more pairs of vertices satisfy the criterion → longer calculation, possibly more accurate result.",
                "Minimum angle",
                0, 
                new ImageIcon(getClass().getResource("/angle.png"))
        );
    }
    
    private void showNormalAngleHelp() {                                             
        JOptionPane.showMessageDialog(WindowManager.getDefault().findTopComponent("SingleFaceTab"),
                "Entered number represents how large the angle between normal vector of candidate plane and vector" + System.lineSeparator()
                        + "from subtraction of normal vectors of two vertices can be to take into account these vertices while counting the approximate symmetry." + System.lineSeparator()
                         + System.lineSeparator()
                        + "Higher number → fewer pairs of vertices satisfy the criterion → shorter calculation, possibly less accurate result." + System.lineSeparator()
                        + "Lower number → more pairs of vertices satisfy the criterion → longer calculation, possibly more accurate result.",
                "Minimum normal angle",
                0, 
                new ImageIcon(getClass().getResource("/angle.png"))
        );
    }   
    
    private void showMinCurvHelp() {                                         
        JOptionPane.showMessageDialog(WindowManager.getDefault().findTopComponent("SingleFaceTab"),
                "Entered number represents how similar the curvature in two vertices must be" + System.lineSeparator()
                        + "to take into account these vertices while counting the plane of approximate symmetry." + System.lineSeparator()
                        + "The higher the number is the more similar they must be." + System.lineSeparator()
                        + System.lineSeparator()
                        + "Higher number → fewer pairs of vertices satisfy the criterion → shorter calculation, possibly less accurate result." + System.lineSeparator()
                        + "Lower number → more pairs of vertices satisfy the criterion → longer calculation, possibly more accurate result.",
                "Minimum curvature ratio",
                0, 
                new ImageIcon(getClass().getResource("/curvature.png"))
        );
    }                                        

    private void showRelDistHelp() {
        JOptionPane.showMessageDialog(WindowManager.getDefault().findTopComponent("SingleFaceTab"),
                "Entered number represents how far middle point of two vertices can be from candidate plane of symmetry" + System.lineSeparator()
                        + "to give this plane vote. Plane with highest number of votes is plane of approximate symmetry." + System.lineSeparator()
                        + System.lineSeparator()
                        + "Higher number → more pairs of vertices satisfy the criterion → longer calculation, possibly more accurate result." + System.lineSeparator()
                        + "Lower number → fewer pairs of vertices satisfy the criterion → shorter calculation, possibly less accurate result.",
                "Maximum relative distance from plane",
                0, 
                new ImageIcon(getClass().getResource("/distance.png"))
        );
    }
    
    private void showAveragingHelp() {
        JOptionPane.showMessageDialog(WindowManager.getDefault().findTopComponent("SingleFaceTab"),
                "TO DO",
                "TO DO",
                0, 
                new ImageIcon(getClass().getResource("/distance.png"))
        );
    }
}
